@@ -42,23 +42,24 @@
 #include <ofono/lte.h>
 #include <ofono/voicecall.h>
 
-#include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
 
+#include <drivers/ubloxmodem/ubloxmodem.h>
+
+static const char *uusbconf_prefix[] = { "+UUSBCONF:", NULL };
 static const char *none_prefix[] = { NULL };
 
-enum supported_models {
-	SARA_G270			= 1102,
-	TOBYL2_COMPATIBLE_MODE		= 1141,
-	TOBYL2_MEDIUM_THROUGHPUT_MODE	= 1143,
-	TOBYL2_HIGH_THROUGHPUT_MODE	= 1146,
+enum ublox_device_flags {
+	UBLOX_DEVICE_F_HIGH_THROUGHPUT_MODE	= (1 << 0),
 };
 
 struct ublox_data {
 	GAtChat *modem;
 	GAtChat *aux;
-	int model_id;
 	enum ofono_vendor vendor_family;
+
+	const struct ublox_model *model;
+	int flags;
 };
 
 static void ublox_debug(const char *str, void *user_data)
@@ -144,77 +145,151 @@ static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	if (data->model_id == TOBYL2_HIGH_THROUGHPUT_MODE)
-		/* use bridged mode until routed mode support is added */
-		g_at_chat_send(data->aux, "AT+UBMCONF=2", none_prefix,
-						NULL, NULL, NULL);
-
 	ofono_modem_set_powered(modem, TRUE);
 }
 
+static void query_usbconf_cb(gboolean ok,
+				GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct ublox_data *data = ofono_modem_get_data(modem);
+	GAtResultIter iter;
+	int profile;
+
+	if (!ok) {
+		ofono_error("Unable to query USB configuration");
+		goto error;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+retry:
+	if (!g_at_result_iter_next(&iter, "+UUSBCONF:")) {
+		ofono_error("Unable to query USB configuration");
+		goto error;
+	}
+
+	if (!g_at_result_iter_next_number(&iter, &profile))
+		goto retry;
+
+	switch (profile) {
+	case 0: /* Fairly back compatible */
+	case 1: /* Fairly back compatible plus audio */
+		break;
+	case 2: /* Low/medium throughput */
+		ofono_error("Medium throughput mode not supported");
+		goto error;
+	case 3: /* High throughput mode */
+		data->flags |= UBLOX_DEVICE_F_HIGH_THROUGHPUT_MODE;
+		break;
+	default:
+		ofono_error("Unexpected USB profile: %d", profile);
+		goto error;
+	}
+
+	if (g_at_chat_send(data->aux, "AT+CFUN=4", none_prefix,
+					cfun_enable, modem, NULL))
+		return;
+
+error:
+	g_at_chat_unref(data->aux);
+	data->aux = NULL;
+	g_at_chat_unref(data->modem);
+	data->modem = NULL;
+	ofono_modem_set_powered(modem, FALSE);
+}
+
+static void query_model_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct ublox_data *data = ofono_modem_get_data(modem);
+	struct ofono_error error;
+	const char *model;
+	const struct ublox_model *m;
+
+	decode_at_error(&error, g_at_result_final_response(result));
+
+	if (!ok)
+		goto fail;
+
+	if (at_util_parse_attr(result, "", &model) == FALSE) {
+		ofono_error("Failed to query modem model");
+		goto fail;
+	}
+
+	m = ublox_model_from_name(model);
+	if (!m) {
+		ofono_error("Unrecognized model: %s", model);
+		goto fail;
+	}
+
+	data->model = m;
+
+	DBG("Model: %s", data->model->name);
+
+	data->vendor_family = OFONO_VENDOR_UBLOX;
+
+	if (data->model->flags & UBLOX_F_HAVE_USBCONF) {
+		if (g_at_chat_send(data->aux, "AT+UUSBCONF?", uusbconf_prefix,
+					query_usbconf_cb, modem, NULL))
+			return;
+
+		ofono_error("Unable to query USB configuration");
+		goto fail;
+	}
+
+	if (g_at_chat_send(data->aux, "AT+CFUN=4", none_prefix,
+					cfun_enable, modem, NULL))
+		return;
+
+fail:
+	g_at_chat_unref(data->aux);
+	data->aux = NULL;
+	g_at_chat_unref(data->modem);
+	data->modem = NULL;
+	ofono_modem_set_powered(modem, FALSE);
+}
+
 static int ublox_enable(struct ofono_modem *modem)
 {
 	struct ublox_data *data = ofono_modem_get_data(modem);
-	const char *model_str = NULL;
 
 	DBG("%p", modem);
 
-	model_str = ofono_modem_get_string(modem, "Model");
-	if (model_str == NULL)
-		return -EINVAL;
-
-	/*
-	 * Toby L2 devices are more complex and special than previously
-	 * supported U-Blox devices. So they need a vendor of their own.
+	data->aux = open_device(modem, "Aux", "Aux: ");
+	/* If this is a serial modem then the device may be behind
+	 * the 'Device' attribute instead...
 	 */
-	data->model_id = atoi(model_str);
-
-	switch (data->model_id) {
-	case SARA_G270:
-		data->vendor_family = OFONO_VENDOR_UBLOX;
-		break;
-	case TOBYL2_COMPATIBLE_MODE:
-	case TOBYL2_HIGH_THROUGHPUT_MODE:
-		data->vendor_family = OFONO_VENDOR_UBLOX_TOBY_L2;
-		break;
-	case TOBYL2_MEDIUM_THROUGHPUT_MODE:
-		DBG("low/medium throughtput profile unsupported");
-		break;
-	default:
-		DBG("unknown ublox model id %d", data->model_id);
-		return -EINVAL;
+	if (data->aux == NULL) {
+		data->aux = open_device(modem, "Device", "Aux: ");
+		if (data->aux == NULL)
+			return -EINVAL;
 	}
 
-	data->aux = open_device(modem, "Aux", "Aux: ");
-	if (data->aux == NULL)
-		return -EINVAL;
-
-	if (data->vendor_family == OFONO_VENDOR_UBLOX) {
-		data->modem = open_device(modem, "Modem", "Modem: ");
-		if (data->modem == NULL) {
-			g_at_chat_unref(data->aux);
-			data->aux = NULL;
-			return -EIO;
-		}
-
+	data->modem = open_device(modem, "Modem", "Modem: ");
+	if (data->modem) {
 		g_at_chat_set_slave(data->modem, data->aux);
-
 		g_at_chat_send(data->modem, "ATE0 +CMEE=1", none_prefix,
 						NULL, NULL, NULL);
-
 		g_at_chat_send(data->modem, "AT&C0", NULL, NULL, NULL, NULL);
 	}
 
 	/* The modem can take a while to wake up if just powered on. */
 	g_at_chat_set_wakeup_command(data->aux, "AT\r", 1000, 11000);
 
-	g_at_chat_send(data->aux, "ATE0 +CMEE=1", none_prefix,
+	g_at_chat_send(data->aux, "ATE0", none_prefix,
+					NULL, NULL, NULL);
+	g_at_chat_send(data->aux, "AT+CMEE=1", none_prefix,
 					NULL, NULL, NULL);
 
-	g_at_chat_send(data->aux, "AT+CFUN=4", none_prefix,
-					cfun_enable, modem, NULL);
+	if (g_at_chat_send(data->aux, "AT+CGMM", NULL,
+				query_model_cb, modem, NULL) > 0)
+		return -EINPROGRESS;
 
-	return -EINPROGRESS;
+	g_at_chat_unref(data->aux);
+	data->aux = NULL;
+
+	return -EINVAL;
 }
 
 static void cfun_disable(gboolean ok, GAtResult *result, gpointer user_data)
@@ -308,19 +383,34 @@ static void ublox_post_sim(struct ofono_modem *modem)
 	struct ofono_gprs *gprs;
 	struct ofono_gprs_context *gc;
 	GAtChat *chat = data->modem ? data->modem : data->aux;
-	const char *driver = data->model_id == TOBYL2_HIGH_THROUGHPUT_MODE ?
-						"ubloxmodem" : "atmodem";
+	const char *driver;
 	/* Toby L2: Create same number of contexts as supported PDP contexts. */
-	int ncontexts = data->model_id == TOBYL2_HIGH_THROUGHPUT_MODE ? 8 : 1;
+	int ncontexts = data->flags & UBLOX_DEVICE_F_HIGH_THROUGHPUT_MODE ? 8 : 1;
+	int variant;
 
 	DBG("%p", modem);
 
 	gprs = ofono_gprs_create(modem, data->vendor_family, "atmodem",
 					data->aux);
 
+	if (ublox_is_toby_l4(data->model)) {
+		driver = "ubloxmodem";
+		variant = ublox_model_to_id(data->model);
+	} else if (ublox_is_toby_l2(data->model)) {
+		if (data->flags & UBLOX_DEVICE_F_HIGH_THROUGHPUT_MODE) {
+			driver = "ubloxmodem";
+			variant = ublox_model_to_id(data->model);
+		} else {
+			driver = "atmodem";
+			variant = OFONO_VENDOR_UBLOX;
+		}
+	} else {
+		driver = "atmodem";
+		variant = OFONO_VENDOR_UBLOX;
+	}
+
 	while (ncontexts) {
-		gc = ofono_gprs_context_create(modem, data->vendor_family,
-						driver, chat);
+		gc = ofono_gprs_context_create(modem, variant, driver, chat);
 
 		if (gprs && gc)
 			ofono_gprs_add_context(gprs, gc);
@@ -328,7 +418,8 @@ static void ublox_post_sim(struct ofono_modem *modem)
 		--ncontexts;
 	}
 
-	ofono_lte_create(modem, 0, "ubloxmodem", data->aux);
+	ofono_lte_create(modem,
+		ublox_model_to_id(data->model), "ubloxmodem", data->aux);
 }
 
 static void ublox_post_online(struct ofono_modem *modem)
